@@ -12,7 +12,21 @@ import logging
 import os
 from datetime import datetime
 from functools import wraps
-from typing import Callable, Dict, Iterable, Sequence, Union
+from typing import TYPE_CHECKING, Callable, Dict, Iterable, Optional, Tuple, Union, cast
+import getpass
+import warnings
+import subprocess
+import sys
+
+
+if TYPE_CHECKING:
+    # https://github.com/microsoft/pyright/issues/1358
+    from typing import TypedDict
+else:
+    try:
+        from typing import TypedDict
+    except ImportError:
+        from typing_extensions import TypedDict
 
 from requests.sessions import Session
 
@@ -31,9 +45,22 @@ except ImportError:
     pass
 
 
-_SESSION_STUFF = {
-    "calcbench_user_name": os.environ.get("CALCBENCH_USERNAME"),
-    "calcbench_password": os.environ.get("CALCBENCH_PASSWORD"),
+class _SESSION_VARIABLES(TypedDict):
+    calcbench_user_name: Optional[str]
+    calcbench_password: Optional[str]
+    api_url_base: str
+    logon_url: str
+    domain: str
+    session: Optional[Session]
+    ssl_verify: bool
+    timeout: int
+    enable_backoff: bool
+    proxies: Optional[Dict[str, str]]
+
+
+_SESSION_STUFF: _SESSION_VARIABLES = {
+    "calcbench_user_name": None,
+    "calcbench_password": None,
     "api_url_base": "https://www.calcbench.com/api/{0}",
     "logon_url": "https://www.calcbench.com/account/LogOnAjax",
     "domain": "https://www.calcbench.com/{0}",
@@ -44,21 +71,99 @@ _SESSION_STUFF = {
     "proxies": None,
 }
 
+KEYRING_SERVICE_NAME = "calcbench_api"
+
+USERNAME_ENVIRONMENT_VARIABLE = "CALCBENCH_USERNAME"
+PASSWORD_ENVIRONMENT_VARIABLE = "CALCBENCH_PASSWORD"
+
+
+def _get_credentials() -> Tuple[str, str]:
+    """
+    Used to store credentials as environment variables.  Now we use keyring
+
+    """
+    session_user_name = _SESSION_STUFF.get("calcbench_user_name")
+    session_password = _SESSION_STUFF.get("calcbench_password")
+    if session_user_name and session_password:
+        return (session_user_name, session_password)
+    keyring_found = False
+    try:
+        import keyring
+
+        keyring_found = True
+        logger.info("found keyring module")
+    except ImportError:
+        logger.info("did not find keyring module")
+    else:
+        try:
+            keychain_credentials = keyring.get_credential(KEYRING_SERVICE_NAME, None)
+        except Exception as e:
+            warnings.warn(f"Exception getting credentials from keyring {e}")
+        else:
+            if keychain_credentials:
+                user_name = cast(str, keychain_credentials.username)
+                password = cast(str, keychain_credentials.password)
+                logger.info(f"found credentials for {user_name} in keyring")
+                _SESSION_STUFF["calcbench_user_name"] = user_name
+                _SESSION_STUFF["calcbench_password"] = password
+                return (user_name, password)
+    environment_user_name = os.environ.get(USERNAME_ENVIRONMENT_VARIABLE)
+    environment_password = os.environ.get(PASSWORD_ENVIRONMENT_VARIABLE)
+    if environment_user_name and environment_password:
+        _SESSION_STUFF["calcbench_user_name"] = environment_user_name
+        _SESSION_STUFF["calcbench_password"] = environment_password
+        logger.info(
+            f"found credentials for {environment_user_name} in environment variables"
+        )
+        return (cast(str, environment_user_name), cast(str, environment_password))
+    else:
+        user_name = input("Calcbench username/email::")
+        password = getpass.getpass("Calcbench password::")
+
+        _SESSION_STUFF["calcbench_user_name"] = user_name
+        _SESSION_STUFF["calcbench_password"] = password
+        store_in_keychain = (
+            input(
+                f"Store credentials in keychain? (yes/no).  {'' if keyring_found else 'Will attempt to install the keyring package.'}"
+            ).lower()[0]
+            == "y"
+        )
+
+        if store_in_keychain:
+            if not keyring_found:
+                try:
+                    subprocess.check_call(
+                        [sys.executable, "-m", "pip", "install", "keyring"]
+                    )
+                except Exception as e:
+                    logger.error("exception installing keyring module")
+                else:
+                    logger.info("successfully install keyring package")
+                    keyring_found = True
+            if keyring_found:
+                import keyring
+
+                try:
+                    keyring.set_password(
+                        KEYRING_SERVICE_NAME, username=user_name, password=password
+                    )
+
+                except Exception as e:
+                    warnings.warn(f"Exception saving credentials to keyring {e}")
+                else:
+                    logger.info("saved credentials to keyring")
+        else:
+            print(
+                f'To avoid this prompt: save credentials to keychain, call "cb.set_credentials()" or set the "{USERNAME_ENVIRONMENT_VARIABLE}" and "{PASSWORD_ENVIRONMENT_VARIABLE}" environment variable'
+            )
+        return (user_name, password)
+
 
 def _calcbench_session() -> Session:
     session = _SESSION_STUFF.get("session")
     if not session:
-        user_name = _SESSION_STUFF.get("calcbench_user_name")
-        password = _SESSION_STUFF.get("calcbench_password")
-        if not (user_name and password):
-            import getpass
+        user_name, password = _get_credentials()
 
-            user_name = input(
-                'Calcbench username/email. Set the "calcbench_user_name" environment variable or call "set_credentials" to avoid this prompt::'
-            )
-            password = getpass.getpass(
-                'Calcbench password.  Set the "calcbench_password" enviroment variable to avoid this prompt::'
-            )
         session = requests.Session()
         if _SESSION_STUFF.get("proxies"):
             session.proxies.update(_SESSION_STUFF["proxies"])
@@ -70,6 +175,8 @@ def _calcbench_session() -> Session:
         )
         r.raise_for_status()
         if r.text != "true":
+            _SESSION_STUFF["calcbench_user_name"] = None
+            _SESSION_STUFF["calcbench_password"] = None
             raise ValueError(
                 "Incorrect Credentials, use the email and password you use to login to Calcbench."
             )
@@ -111,11 +218,12 @@ def _add_backoff(f):
 @_add_backoff
 def _json_POST(end_point: str, payload: Union[dict, APIQueryParams]):
     url = _SESSION_STUFF["api_url_base"].format(end_point)
-    logger.debug(f"posting to {url}, {payload}")
+    data = json.dumps(payload)
+    logger.debug(f"posting to {url}, {data}")
     start = datetime.now()
     response = _calcbench_session().post(
         url,
-        data=json.dumps(payload),
+        data=data,
         headers={"content-type": "application/json"},
         verify=_SESSION_STUFF["ssl_verify"],
         timeout=_SESSION_STUFF["timeout"],
@@ -154,7 +262,7 @@ def set_credentials(cb_username: str, cb_password: str):
 
     Call this before any other Calcbench functions.
 
-    Alternatively set the ``CALCBENCH_USERNAME`` and ``CALCBENCH_PASSWORD`` environment variables
+    Alternatively set the ``CALCBENCH_USERNAME`` and ``CALCBENCH_PASSWORD`` environment variables OR set the calcbench_api credentials on your Keychain|Secret Service|Windows Credential Locker
 
     :param str cb_username: Your calcbench.com email address
     :param str cb_password: Your calcbench.com password
@@ -170,7 +278,8 @@ def set_credentials(cb_username: str, cb_password: str):
 
 
 def enable_backoff(
-    backoff_on: bool = True, giveup: Callable[[Exception], bool] = lambda e: False
+    backoff_on: bool = True,
+    giveup: Callable[[Exception], bool] = lambda e: e.response.status_code == 404,
 ):
     """Re-try failed requests with exponential back-off
 
@@ -178,6 +287,8 @@ def enable_backoff(
 
     If processes make many requests, failures are inevitable.  Call this to retry failed requests.
 
+    By default gives up immediately if the server returns 404
+
     :param backoff_on: toggle backoff
     :param giveup: function that handles exception and decides whether to continue or not.
 
