@@ -4,6 +4,13 @@ from datetime import date, datetime
 from enum import Enum
 from typing import Generator, Iterable, Optional, Sequence
 
+from calcbench.api_query_params import (
+    CompanyIdentifiers,
+    Period,
+    PeriodArgument,
+    PeriodType,
+)
+
 try:
     from typing import Literal
 except ImportError:
@@ -16,14 +23,11 @@ except ImportError:
 
 
 from calcbench.api_client import (
-    CompanyIdentifiers,
-    Period,
-    PeriodArgument,
-    PeriodType,
     _json_GET,
     _json_POST,
     _try_parse_timestamp,
     logger,
+    set_field_values,
 )
 
 try:
@@ -66,7 +70,7 @@ def disclosure_dataframe(
     :param identifier_key: how to index the returned DataFrame.
     :param use_fiscal_period: Index disclosure by fiscal, as opposed to calendar periods.
     :param entire_universe: Data for all companies
-    :return: A DataFrame of DislcosureSearchResults indexed by document name -> company identifier.
+    :return: A DataFrame of DisclosureSearchResults indexed by document name -> company identifier.
 
     Usage::
 
@@ -78,7 +82,7 @@ def disclosure_dataframe(
 
     """
     if block_tag_names:
-        docs: Iterable[DislcosureSearchResults] = []
+        docs: Iterable[DisclosureSearchResults] = []
         for block_tag_name in block_tag_names:
             docs.extend(
                 disclosure_search(
@@ -213,10 +217,7 @@ class DisclosureContent:
     CIK: str
 
     def __init__(self, **kwargs):
-        names = set([f.name for f in dataclasses.fields(self)])
-        for k, v in kwargs.items():
-            if k in names:
-                setattr(self, k, v)
+        set_field_values(self, kwargs)
 
     @property
     def contents(self) -> str:
@@ -236,7 +237,7 @@ PERIOD_MAP = {
 
 
 @dataclass
-class DislcosureSearchResults(dict):
+class DisclosureSearchResults(dict):
     """
     Represents a disclosure.
     """
@@ -341,7 +342,7 @@ def disclosure_search(
     progress_bar: "tqdm.std.tqdm" = None,
     accession_id: int = None,
     all_text_blocks: bool = False,
-) -> Generator[DislcosureSearchResults, None, None]:
+) -> Generator[DisclosureSearchResults, None, None]:
     """
     Footnotes and other text
 
@@ -450,7 +451,7 @@ def _document_search_results(payload, progress_bar=None):
         if progress_bar is not None:
             progress_bar.update(len(disclosures))
         for result in disclosures:
-            yield DislcosureSearchResults(**result)
+            yield DisclosureSearchResults(**result)
         payload["pageParameters"]["startOffset"] = results["nextGroupStartOffset"]
     payload["pageParameters"]["startOffset"] = None
 
@@ -472,4 +473,4 @@ def _document_by_block_tag_name(
 ) -> DisclosureContent:
     payload = {"accession_ids": accession_id, "block_tag_name": block_tag_name}
     json = _json_GET("query/disclosuresByTag", payload)
-    return DisclosureContent(**json[0])
\ No newline at end of file
+    return DisclosureContent(**json[0])
