@@ -1,17 +1,55 @@
-import logging
 from typing import Callable, Sequence, TypeVar
 
 
 import calcbench as cb
 import pandas as pd
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
-logging.getLogger("calcbench").setLevel(logging.DEBUG)
-cb.enable_backoff(giveup=lambda e: e.response.status_code == 404)
+cb.enable_backoff(giveup=lambda e: e.response.status_code in [404, 500])
 
 T = TypeVar("T")
 
 
+def iterate_to_dataframe(
+    arguments: Sequence[T],
+    f: Callable[[T], pd.DataFrame],
+):
+    """Apply arguments to a function that returns a DataFrame append to a dataframe and return.
+
+    Usage::
+
+    >>> %pip install calcbench-api-client[Pandas,Backoff,tqdm]
+    >>> from calcbench.downloaders import iterate_to_dataframe
+    >>> import calcbench as cb
+    >>> tickers = cb.tickers(entire_universe=True)
+    >>> d = iterate_and_save_pandas(
+    >>>    tickers,
+    >>>    lambda ticker: cb.point_in_time(
+    >>>        all_face=True,
+    >>>        all_footnotes=False,
+    >>>        company_identifiers=[ticker],
+    >>>        all_history=True,
+    >>>        include_preliminary=True,
+    >>>        include_xbrl=True,
+    >>>    ),
+    >>> )
+    """
+    argument: T
+    chunks = []
+    for argument in tqdm(list(arguments)):
+        try:
+            d = f(argument)
+            if d.empty:
+                continue
+        except KeyboardInterrupt:
+            raise
+        except Exception as e:
+            tqdm.write(f"Exception getting {argument} {e}")
+        else:
+            chunks.append(d)
+    return pd.concat(chunks)
+
+
 def iterate_and_save_pandas(
     arguments: Sequence[T], f: Callable[[T], pd.DataFrame], file_name: str
 ):
@@ -40,7 +78,7 @@ def iterate_and_save_pandas(
     argument: T
     write_mode = "w"
     write_headers = True
-    for argument in tqdm(arguments):
+    for argument in tqdm(list(arguments)):
         try:
             data = f(argument)
             if data.empty:
