@@ -183,6 +183,11 @@ def standardized_raw(
             "specifying metrics with all_face or all_footnotes does not make sense"
         )
 
+    if not any([all_face, all_footnotes, metrics]):
+        raise ValueError(
+            "need to specify a metrics argument, 'all_face', 'all_foonotes', or a metric"
+        )
+
     if isinstance(metrics, str):
         raise TypeError("metrics should be a list of strings")
 
@@ -262,10 +267,20 @@ def point_in_time(
     accession_id: int = None,
     include_trace: bool = False,
     set_index: bool = False,
+    _point_in_time_mode=True,
 ) -> "pd.DataFrame":
     """Point-in-Time Data
 
-    Standardized data with a timestamp when it was published by Calcbench
+    Standardized data with a timestamp when it was published by Calcbench.
+
+
+    A record is returned for each filing in which a metric value changed (was revised).
+
+    If the company files an 8-K with revenue = $100 then a week later files a 10-K with revenue = $100 one record will be returned for that period.  It will have preliminary=True and XBRL=True.
+
+    If the company files an 8-K with revenue = $100 and then a week later files a 10-K with revenue = $200, two records will be returned.  One with revenue = $100, a revision_number=0, date_reported of 8-K, preliminary=True and XBRL=False.  You will see a second line with revenue = $200, a revision_number=1, and date_reported of the 10-K, preliminary=False and XBRL=False.
+
+    If the value is revised in subsequent XBRL filings you will see a record for each filing with an incremented revision number.
 
     :param accession_id: Unique identifier for the filing for which to recieve data.  Pass this to recieve data for one filing.  Same as filing_id in filings objects
     :param all_face: Retrieve all of the points from the face financials, income/balance/statement of cash flows
@@ -274,7 +289,7 @@ def point_in_time(
     :param include_xbrl: Include facts from XBRL 10-K/Qs.
     :param include_trace: Include a URL that points to the source document.
     :param set_index: Set a useful index on the returned DataFrame
-    :param exclude_errors:  Run the error detection
+    :param _point_in_time_mode: DO NOT USE.  For debugging only.
     :return: DataFrame of facts
 
     Columns:
@@ -293,7 +308,7 @@ def point_in_time(
     period_end
        Last day of the fiscal period for this fact
     date_reported
-       Timestamp when Calcbench published this fact.
+       Timestamp (EST) when Calcbench published this fact.
 
        In some cases, particularly prior to 2015, this will be the filing date of the document as recorded by the SEC.  To exclude these points remove points where the hour is 0.
     metric
@@ -332,7 +347,7 @@ def point_in_time(
         company_identifiers=company_identifiers,
         all_face=all_face,
         all_footnotes=all_footnotes,
-        point_in_time=True,
+        point_in_time=_point_in_time_mode,
         metrics=metrics,
         all_history=all_history,
         entire_universe=entire_universe,
